local UIAnim = require "widgets/uianim"
local Text = require "widgets/text"
local Image = require "widgets/image"
local Widget = require "widgets/widget"
local CacheItem = require "iteminfo_cacheitem"

local easing = require("easing")

local ItemInfoDesc = Class(Widget, function(self, slot)
	Widget._ctor(self, "ItemInfoDesc")
	
	self.slot = slot
	self.container = nil -- Set from containerwidget.Open, hooked from modmain

	self.width = 0
	self.height = 0
	
	self.relative_scale = 1
	
	self.num_rows = 0
	
	self.time_since_last_update = 0
	
	self.active = false
	
	self:SetClickable(false)
	end)

local function Format1D(value)
	return math.floor(value * 10 + 0.5) / 10
end

local function FormatInt(value)
	return math.floor(value + 0.5)
end

local function FormatTime(t)
	if MOD_ITEMINFO.TIME_FORMAT == MOD_ITEMINFO.TIME_FORMATS.HOURS then
		if t == 0 then
			return "0"
		else
			local	hours = string.format("%01.f", math.floor(t/3600))
			local	mins = string.format("%02.f", math.floor(t/60 - (hours*60)))
			local	secs = string.format("%02.f", math.floor(t - hours*3600 - mins *60))
			
			if hours == "0" and mins == "00" then
				return secs
				elseif hours == "0" then
					return mins .. ":" .. secs
				else
					return hours .. ":" .. mins .. ":" .. secs
				end

			end
			elseif MOD_ITEMINFO.TIME_FORMAT == MOD_ITEMINFO.TIME_FORMATS.DAYS then
				if t == 0 then
					return "0d"
				else
					return Format1D(t/TUNING.TOTAL_DAY_TIME) .. "d"
				end
			end

			return "???"
		end

		function ItemInfoDesc:GetSpoilModifier(inst, inst_owner)
			local modifier = 1.0
			local owner = inst.components.inventoryitem and inst.components.inventoryitem.owner  or inst_owner

			if owner then
				if owner:HasTag("fridge") then
					if inst:HasTag("frozen") then
						modifier = TUNING.PERISH_COLD_FROZEN_MULT
					else
						modifier = TUNING.PERISH_FRIDGE_MULT
					end
					elseif owner:HasTag("spoiler") then
						modifier = TUNING.PERISH_GROUND_MULT 
					end
				else
					modifier = TUNING.PERISH_GROUND_MULT 
				end

				if inst.components.moisturelistener and inst.components.moisturelistener:IsWet() then
					modifier = modifier * TUNING.PERISH_WET_MULT
				end
				local season = GetSeasonManager()
				if season and season:GetCurrentTemperature() < 0 then
					if inst:HasTag("frozen") and not inst.components.perishable.frozenfiremult then
						modifier = TUNING.PERISH_COLD_FROZEN_MULT
					else
						modifier = modifier * TUNING.PERISH_WINTER_MULT
					end
				end
				if inst.components.perishable and inst.components.perishable.frozenfiremult then
					modifier = modifier * TUNING.PERISH_FROZEN_FIRE_MULT
				end
				if TUNING.OVERHEAT_TEMP ~= nil and season and season:GetCurrentTemperature() > TUNING.OVERHEAT_TEMP then
					modifier = modifier * TUNING.PERISH_SUMMER_MULT
				end

				modifier = modifier * TUNING.PERISH_GLOBAL_MULT

				return modifier
			end

			function ItemInfoDesc:GetPerishFreshness(inst)
				local percent = 1
				if inst.components.perishable then
					percent = inst.components.perishable:GetPercent()
				end
				return percent
			end

			function ItemInfoDesc:CalculateFinalPerishValues(base_inst, inst, owner)
				local freshness = self:GetPerishFreshness(inst)
				local perishtime = base_inst.components.perishable.perishtime
				local max_perishtime = perishtime

				local modifier = self:GetSpoilModifier(inst, owner)

				perishtime = perishtime * freshness / modifier
				max_perishtime = max_perishtime / modifier

				return freshness, perishtime, max_perishtime
			end

			function ItemInfoDesc:GetPerishValues(base_inst, inst)
				if self.container ~= nil then
					return self:CalculateFinalPerishValues(base_inst, inst, self.container)
				end

				return self:CalculateFinalPerishValues(base_inst, inst, GetPlayer())
			end

			function ItemInfoDesc:GetDappernessValue(base_inst, inst)
				return base_inst.components.equippable.dapperness * 60
			end

			function ItemInfoDesc:GetFuelValue(base_inst, inst)
				local maxfuel = base_inst.components.fueled.maxfuel
				local percentused = inst.components.fueled:GetPercent()

				local remaining_time = maxfuel * percentused

				if inst.prefab == "torch" then
					if GetSeasonManager():IsRaining() then 
						remaining_time = remaining_time / (1 + TUNING.TORCH_RAIN_RATE * GetSeasonManager():GetPrecipitationRate())
					end
				end

				local fueltype = base_inst.components.fueled.fueltype

				return remaining_time, fueltype
			end

			function ItemInfoDesc:CanEat(base_inst)
				local foodtype = base_inst.components.edible.foodtype

				if foodtype == "GEARS" then
					if GetPlayer():HasTag("GEARS_eater") then return true else return false end
					elseif foodtype == "WOOD" then
						if GetPlayer():HasTag("WOOD_eater") then return true else return false end
						elseif foodtype == "ELEMENTAL" then
							if GetPlayer():HasTag("ELEMENTAL_eater") then return true else return false end
							elseif foodtype == "HORRIBLE" then
								if GetPlayer():HasTag("HORRIBLE_eater") then return true else return false end
								elseif foodtype == "BURNT" then
									if GetPlayer():HasTag("BURNT_eater") then return true else return false end
									elseif foodtype == "RAW" then
										if GetPlayer():HasTag("RAW_eater") then return true else return false end
										elseif foodtype == "ROUGHAGE" then
											if GetPlayer():HasTag("ROUGHAGE_eater") then return true else return false end
										end

										return true
									end

									StrongStomachEaters =
									{
									webber = true
								}

								IgnoreSpoilageEaters =
								{
								wx78 = true
							}

							PickyEaters =
							{
							wickerbottom = true
						}

						NonDegradingFood =
						{
						ice = true
					}

					function ItemInfoDesc:CanEatMonster()
						if StrongStomachEaters[GetPlayer().prefab] then
							return true
						end
					end

					function ItemInfoDesc:IgnoresSpoilage()
						if IgnoreSpoilageEaters[GetPlayer().prefab] then
							return true
						end
					end

					function ItemInfoDesc:IsPickyEater()
						if PickyEaters[GetPlayer().prefab] then
							return true
						end
					end

					function ItemInfoDesc:IsNonDegradingFood(base_inst)
						if NonDegradingFood[base_inst.prefab] then
							return true
						end
					end

					local FOODSTATES = {FRESH = 1, STALE = 2, SPOILED = 3}

					function ItemInfoDesc:GetEdibleValues(base_inst, inst)

						local hunger = base_inst.components.edible.hunger
						local sanity = base_inst.components.edible.sanity
						local health = base_inst.components.edible.health

						local freshness = self:GetPerishFreshness(inst)
						local state = nil

						if freshness >= .5 then
							state = FOODSTATES.FRESH
							elseif freshness > .2 then
								state = FOODSTATES.STALE
							else
								state = FOODSTATES.SPOILED
							end

							if self:CanEatMonster() and inst:HasTag("monstermeat") then
								if health < 0 then health = 0 end
								if sanity < 0 then sanity = 0 end
							end

	-- Hunger
	if not(self:IsNonDegradingFood(base_inst) or hunger < 0 or self:IgnoresSpoilage()) then
		if state == FOODSTATES.STALE then
			hunger = hunger * (self:IsPickyEater() and TUNING.WICKERBOTTOM_STALE_FOOD_HUNGER or TUNING.STALE_FOOD_HUNGER)
			elseif state == FOODSTATES.SPOILED then
				hunger = hunger * (self:IsPickyEater() and TUNING.WICKERBOTTOM_SPOILED_FOOD_HUNGER or TUNING.SPOILED_FOOD_HUNGER)
			end
		end

	-- Sanity
	if not(self:IsNonDegradingFood(base_inst) or sanity < 0 or self:IgnoresSpoilage()) then
		if state == FOODSTATES.STALE then
			sanity = 0
			elseif state == FOODSTATES.SPOILED then
				sanity = -TUNING.SANITY_SMALL
			end
		end

	-- Health
	if not(self:IsNonDegradingFood(base_inst) or health < 0 or self:IgnoresSpoilage()) then
		if state == FOODSTATES.STALE then
			health = health * (self:IsPickyEater() and TUNING.WICKERBOTTOM_STALE_FOOD_HEALTH or TUNING.STALE_FOOD_HEALTH)
			elseif state == FOODSTATES.SPOILED then
				health = health * (self:IsPickyEater() and TUNING.WICKERBOTTOM_SPOILED_FOOD_HEALTH or TUNING.SPOILED_FOOD_HEALTH)
			end
		end


		return hunger, sanity, health
	end

	function ItemInfoDesc:GetPlayerDamageModifier()
		local thePlayer = GetPlayer()
		if thePlayer.components and thePlayer.components.combat then
			if thePlayer.components.combat.damagemultiplier then
				return thePlayer.components.combat.damagemultiplier
				elseif thePlayer.components.combat.GetDamageModifier then
					return thePlayer.components.combat:GetDamageModifier()
				end
			end
			return 1;
		end

		function ItemInfoDesc:GetWeaponDamage(base_inst, inst)

			local damage = 0

			if inst.prefab ~= "hambat" then
				damage = base_inst.components.weapon.damage
			else
				damage = TUNING.HAMBAT_DAMAGE * self:GetPerishFreshness(inst)
				damage = Remap(damage, 0, TUNING.HAMBAT_DAMAGE, TUNING.HAMBAT_MIN_DAMAGE_MODIFIER * TUNING.HAMBAT_DAMAGE, TUNING.HAMBAT_DAMAGE)
			end

			local damagemult = self:GetPlayerDamageModifier()

			return damage, damagemult
		end

		function ItemInfoDesc:GetFiniteUses(base_inst, inst)
			local consumption_per_use = 1
			for k,v in pairs(base_inst.components.finiteuses.consumption) do
				consumption_per_use = v
				break
			end

			local maxuses = base_inst.components.finiteuses.total / consumption_per_use
			local usesleft = inst.components.finiteuses:GetPercent() * maxuses

			return usesleft, maxuses
		end

		function ItemInfoDesc:GetArmorValues(base_inst, inst)
			local absorb_percent = base_inst.components.armor.absorb_percent
			local hp = base_inst.components.armor.maxcondition  * inst.components.armor:GetPercent()

			return absorb_percent, hp
		end

		function ItemInfoDesc:GetInsulationValues(base_inst, inst)
			local insulation = base_inst.components.insulator.insulation
			local insulation_type = base_inst.components.insulator.type

			return insulation, insulation_type
		end

		function ItemInfoDesc:GetWaterproofValue(base_inst, inst)
			return base_inst.components.waterproofer.effectiveness
		end

		function ItemInfoDesc:GetHealerValue(base_inst, inst)
			return base_inst.components.healer.health
		end

-- Spacing between the image and the text
local Spacing = 5

-- Spacing between same row childs
local ChildSpacing = 10

local ImageWidth = 32
local RowHeight = 48

function ItemInfoDesc:AddChild1(img, txt)
	local pos_y = self.num_rows * RowHeight + RowHeight/2

	local txt_w = txt:GetRegionSize()
	
	local width = ImageWidth + txt_w + Spacing
	
	if width > self.width then
		self.width = width
	end
	self.height = self.height + RowHeight
	
	local left = width/2 * -1
	
	img:SetPosition(left + ImageWidth/2, pos_y)
	txt:SetPosition(left + ImageWidth + txt_w/2 + Spacing, pos_y )
	
	self:AddChild(img)
	self:AddChild(txt)
	
	self.num_rows = self.num_rows + 1
end

function ItemInfoDesc:AddChild1_2(img, txt1, txt2, color)
	local pos_y = self.num_rows * RowHeight + RowHeight/2

	local txt1_w = txt1:GetRegionSize()
	local txt2_w = txt2:GetRegionSize()
	
	local width = ImageWidth + txt1_w + txt2_w + Spacing * 1 + 1 + ChildSpacing
	
	if width > self.width then
		self.width = width
	end
	self.height = self.height + RowHeight
	
	local left = width/2 * -1
	
	img:SetPosition(left + ImageWidth/2, pos_y)
	txt1:SetPosition(left + ImageWidth + txt1_w/2 + Spacing, pos_y )
	txt2:SetPosition(left + ImageWidth + txt1_w + txt2_w/2 + Spacing*1 + 1 + ChildSpacing, pos_y )

	txt2.inst.TextWidget:SetColour(color[1], color[2], color[3], color[4])
	
	self:AddChild(img)
	self:AddChild(txt1)
	self:AddChild(txt2)
	
	self.num_rows = self.num_rows + 1
end

function ItemInfoDesc:AddChild2(img1, txt1, img2, txt2)
	local pos_y = self.num_rows * RowHeight + RowHeight/2

	local txt1_w = txt1:GetRegionSize()
	local txt2_w = txt2:GetRegionSize()
	
	local width = ImageWidth * 2 + txt1_w + txt2_w + Spacing * 2 + ChildSpacing
	
	if width > self.width then
		self.width = width
	end
	self.height = self.height + RowHeight
	
	local left = width/2 * -1
	
	img1:SetPosition(left + ImageWidth/2, pos_y)
	txt1:SetPosition(left + ImageWidth + txt1_w/2 + Spacing, pos_y )
	
	img2:SetPosition(left + ImageWidth * 1.5 + txt1_w + Spacing + ChildSpacing, pos_y)
	txt2:SetPosition(left + ImageWidth * 2 + txt1_w + txt2_w/2 + Spacing * 2 + ChildSpacing, pos_y )
	
	self:AddChild(img1)
	self:AddChild(txt1)
	self:AddChild(img2)
	self:AddChild(txt2)
	
	self.num_rows = self.num_rows + 1
end

function ItemInfoDesc:AddChild3(img1, txt1, img2, txt2, img3, txt3)
	local pos_y = self.num_rows * RowHeight + RowHeight/2

	local txt1_w = txt1:GetRegionSize()
	local txt2_w = txt2:GetRegionSize()
	local txt3_w = txt3:GetRegionSize()
	
	local width = ImageWidth * 3 + txt1_w + txt2_w + txt3_w + Spacing * 3 + ChildSpacing * 2
	
	if width > self.width then
		self.width = width
	end
	self.height = self.height + RowHeight
	
	local left = width/2 * -1
	
	img1:SetPosition(left + ImageWidth/2, pos_y)
	txt1:SetPosition(left + ImageWidth + txt1_w/2 + Spacing, pos_y )
	
	img2:SetPosition(left + ImageWidth * 1.5 + txt1_w + Spacing + ChildSpacing, pos_y)
	txt2:SetPosition(left + ImageWidth * 2 + txt1_w + txt2_w/2 + Spacing * 2 + ChildSpacing, pos_y )
	
	img3:SetPosition(left + ImageWidth * 2.5 + txt1_w + txt2_w + Spacing * 2 + ChildSpacing * 2, pos_y )
	txt3:SetPosition(left + ImageWidth * 3 + txt1_w + txt2_w + txt3_w/2 + Spacing * 3 + ChildSpacing * 2, pos_y )
	
	self:AddChild(img1)
	self:AddChild(txt1)
	self:AddChild(img2)
	self:AddChild(txt2)
	self:AddChild(img3)
	self:AddChild(txt3)
	
	self.num_rows = self.num_rows + 1
end

function ItemInfoDesc:AddPerishableRow(base_inst, inst)
	local freshness, perishtime, max_perishtime = self:GetPerishValues(base_inst, inst)
	
	local img1 = Image("images/iteminfo_images.xml", "freshness.tex")
	local txt1 = Text(NUMBERFONT, 42)
	txt1.inst.TextWidget:SetString(Format1D(freshness * 100) .. "%" )
	
	if not base_inst.components.edible or MOD_ITEMINFO.PERISHABLE == MOD_ITEMINFO.PERISH_DISPLAY.PERISH_ONLY then
		local img2 = Image("images/iteminfo_images.xml", "spoiled.tex")
		local txt2 = Text(NUMBERFONT, 42)
		txt2.inst.TextWidget:SetString(FormatTime(perishtime))

		self:AddChild2(img1, txt1, img2, txt2)
		elseif MOD_ITEMINFO.PERISHABLE == MOD_ITEMINFO.PERISH_DISPLAY.STALE_PERISH then
			local stale_time = perishtime - max_perishtime/2

			if stale_time > 0 then
				local img2 = Image("images/iteminfo_images.xml", "stale.tex")
				local txt2 = Text(NUMBERFONT, 42)
				txt2.inst.TextWidget:SetString(FormatTime(stale_time))
				self:AddChild2(img1, txt1, img2, txt2)
			else
				local img2 = Image("images/iteminfo_images.xml", "spoiled.tex")
				local txt2 = Text(NUMBERFONT, 42)
				txt2.inst.TextWidget:SetString(FormatTime(perishtime))
				self:AddChild2(img1, txt1, img2, txt2)
			end
			elseif MOD_ITEMINFO.PERISHABLE == MOD_ITEMINFO.PERISH_DISPLAY.BOTH then
				local img2 = Image("images/iteminfo_images.xml", "spoiled.tex")
				local txt2 = Text(NUMBERFONT, 42)
				txt2.inst.TextWidget:SetString(FormatTime(perishtime))

				self:AddChild2(img1, txt1, img2, txt2)

				local stale_time = perishtime - max_perishtime/2

				if stale_time > 0 then
					local img3 = Image("images/iteminfo_images.xml", "stale.tex")
					local txt3 = Text(NUMBERFONT, 42)
					txt3.inst.TextWidget:SetString(FormatTime(stale_time))

					self:AddChild1(img3, txt3)
				end
			end
		end

		function ItemInfoDesc:AddDappernessRow(base_inst, inst)
			local dapperness = self:GetDappernessValue(base_inst, inst)

			local img = Image("images/iteminfo_images.xml", "sanity.tex")
			local txt = Text(NUMBERFONT, 42)
			txt.inst.TextWidget:SetString(Format1D(dapperness).. "/min")

			self:AddChild1(img, txt)
		end

		function ItemInfoDesc:AddInsulatorRow(base_inst, inst)
			local insulation, insulation_type = self:GetInsulationValues(base_inst, inst)

			local img = Image("images/iteminfo_images.xml", "insulation_" .. insulation_type .. ".tex")
			local txt = Text(NUMBERFONT, 42)
			txt.inst.TextWidget:SetString(insulation)

			self:AddChild1(img, txt)
		end

		function ItemInfoDesc:AddFueledRow(base_inst, inst)
			local fueltime, fueltype = self:GetFuelValue(base_inst, inst)

			local img = Image("images/iteminfo_images.xml","fuel_" .. fueltype .. ".tex")
			local txt = Text(NUMBERFONT, 42)
			txt.inst.TextWidget:SetString(FormatTime(fueltime))

			self:AddChild1(img, txt)
		end

		function ItemInfoDesc:AddWaterproofRow(base_inst, inst)

			local waterproof = self:GetWaterproofValue(base_inst, inst)

			local img = Image("images/iteminfo_images.xml", "waterproof.tex")
			local txt = Text(NUMBERFONT, 42)
			txt.inst.TextWidget:SetString(FormatInt(waterproof * 100) .. "%")

			self:AddChild1(img, txt)
		end

		function ItemInfoDesc:AddWeaponRow(base_inst, inst)
			local damage, damagemult = self:GetWeaponDamage(base_inst, self.item)

			local img = Image("images/iteminfo_images.xml", "weapon.tex")
			local txt = Text(NUMBERFONT, 42)
			local dmg_ori = Format1D(damage)
			txt.inst.TextWidget:SetString(dmg_ori)

			if damagemult < 1 then
				local dmg_div = Format1D(dmg_ori - dmg_ori * damagemult)
				local txt_ext = Text(NUMBERFONT, 35)
				txt_ext.inst.TextWidget:SetString("(-"..dmg_div..")")
				self:AddChild1_2(img, txt, txt_ext, {1,0,0,1})
				elseif damagemult > 1 then
					local dmg_div = Format1D(dmg_ori * damagemult - dmg_ori)
					local txt_ext = Text(NUMBERFONT, 35)
					txt_ext.inst.TextWidget:SetString("(+"..dmg_div..")")
					self:AddChild1_2(img, txt, txt_ext, {0,1,0,1})
				else
					self:AddChild1(img, txt)
				end
			end

			function ItemInfoDesc:AddFiniteUsesRow(base_inst, inst)
				local usesleft, maxuses = self:GetFiniteUses(base_inst, self.item)

				local img = Image("images/iteminfo_images.xml", "uses.tex")
				local txt = Text(NUMBERFONT, 42)
				txt.inst.TextWidget:SetString(FormatInt(usesleft) .. "/" .. maxuses)

				self:AddChild1(img, txt)
			end

			function ItemInfoDesc:AddArmorRow(base_inst, inst)
				local absorb, hp = self:GetArmorValues(base_inst, inst)

				local img1 = Image("images/iteminfo_images.xml", "armor.tex")
				local txt1 = Text(NUMBERFONT, 42)
				txt1.inst.TextWidget:SetString(FormatInt(absorb * 100) .. "%")

				local img2 = Image("images/iteminfo_images.xml", "health.tex")
				local txt2 = Text(NUMBERFONT, 42)
				txt2.inst.TextWidget:SetString(FormatInt(hp))


				self:AddChild2(img1, txt1, img2, txt2)
			end

			function ItemInfoDesc:AddHealerRow(base_inst, inst)

				local health = self:GetHealerValue(base_inst, inst)

				local img = Image("images/iteminfo_images.xml", "health.tex")
				local txt = Text(NUMBERFONT, 42)
				txt.inst.TextWidget:SetString(health)

				self:AddChild1(img, txt)
			end

			function ItemInfoDesc:AddEdibleRow(base_inst, inst)
				local hunger, sanity, health = self:GetEdibleValues(base_inst, inst)

	--[[local img1 = Image("images/iteminfo_images.xml", "hunger.tex")
	local txt1 = Text(NUMBERFONT, 42)
	txt1.inst.TextWidget:SetString(Format1D(hunger))
	
	local img2 = Image("images/iteminfo_images.xml", "sanity.tex")
	local txt2 = Text(NUMBERFONT, 42)
	txt2.inst.TextWidget:SetString(Format1D(sanity))
	
	local img3 = Image("images/iteminfo_images.xml", "health.tex")
	local txt3 = Text(NUMBERFONT, 42)
	txt3.inst.TextWidget:SetString(Format1D(health))]]--
	--self:AddChild3(img1, txt1, img2, txt2, img3, txt3)
	
	if hunger ~= 0 and sanity ~= 0 and health ~= 0 then
		local img1 = Image("images/iteminfo_images.xml", "hunger.tex")
		local txt1 = Text(NUMBERFONT, 42)
		txt1.inst.TextWidget:SetString(Format1D(hunger))
		
		local img2 = Image("images/iteminfo_images.xml", "sanity.tex")
		local txt2 = Text(NUMBERFONT, 42)
		txt2.inst.TextWidget:SetString(Format1D(sanity))
		
		local img3 = Image("images/iteminfo_images.xml", "health.tex")
		local txt3 = Text(NUMBERFONT, 42)
		txt3.inst.TextWidget:SetString(Format1D(health))
		
		self:AddChild3(img1, txt1, img2, txt2, img3, txt3)
		elseif hunger ~= 0 and sanity ~= 0 then
			local img1 = Image("images/iteminfo_images.xml", "hunger.tex")
			local txt1 = Text(NUMBERFONT, 42)
			txt1.inst.TextWidget:SetString(Format1D(hunger))

			local img2 = Image("images/iteminfo_images.xml", "sanity.tex")
			local txt2 = Text(NUMBERFONT, 42)
			txt2.inst.TextWidget:SetString(Format1D(sanity))

			self:AddChild2(img1, txt1, img2, txt2)
			elseif hunger ~= 0 and health ~= 0 then
				local img1 = Image("images/iteminfo_images.xml", "hunger.tex")
				local txt1 = Text(NUMBERFONT, 42)
				txt1.inst.TextWidget:SetString(Format1D(hunger))

				local img3 = Image("images/iteminfo_images.xml", "health.tex")
				local txt3 = Text(NUMBERFONT, 42)
				txt3.inst.TextWidget:SetString(Format1D(health))

				self:AddChild2(img1, txt1, img3, txt3)
				elseif sanity ~= 0 and health ~= 0 then
					local img2 = Image("images/iteminfo_images.xml", "sanity.tex")
					local txt2 = Text(NUMBERFONT, 42)
					txt2.inst.TextWidget:SetString(Format1D(sanity))

					local img3 = Image("images/iteminfo_images.xml", "health.tex")
					local txt3 = Text(NUMBERFONT, 42)
					txt3.inst.TextWidget:SetString(Format1D(health))

					self:AddChild2(img2, txt2, img3, txt3)
					elseif hunger ~= 0 then 
						local img1 = Image("images/iteminfo_images.xml", "hunger.tex")
						local txt1 = Text(NUMBERFONT, 42)
						txt1.inst.TextWidget:SetString(Format1D(hunger))

						self:AddChild1(img1, txt1)
						elseif sanity ~= 0 then 
							local img2 = Image("images/iteminfo_images.xml", "sanity.tex")
							local txt2 = Text(NUMBERFONT, 42)
							txt2.inst.TextWidget:SetString(Format1D(sanity))

							self:AddChild1(img2, txt2)
							elseif health ~= 0 then 
								local img3 = Image("images/iteminfo_images.xml", "health.tex")
								local txt3 = Text(NUMBERFONT, 42)
								txt3.inst.TextWidget:SetString(Format1D(health))

								self:AddChild1(img3, txt3)
							end
						end

						function ItemInfoDesc:AddPrefabNameRow(inst)
							local name = inst.prefab

							local img = Image("images/iteminfo_images.xml", "label.tex")
							local txt = Text(NUMBERFONT, 42)
							txt.inst.TextWidget:SetString(name)

							self:AddChild1(img, txt)
						end

						local IgnoredPrefabs =
						{
						blueprint = true,
						heatrock = true,
						fireflies = true
					}

					function ItemInfoDesc:ShouldBeIgnored(inst)
						if IgnoredPrefabs[inst.prefab] then return true end
						return false
					end

					function ItemInfoDesc:UpdateInfo()
						self.num_rows = 0

						self.width = 0
						self.height = 0

						if not self.item or not self.item.components or not self.item.components.inventoryitem then return end

						if not self:ShouldBeIgnored(self.item) then
							if not MOD_ITEMINFO.CACHED_ITEMS[self.item.prefab] then

								MOD_ITEMINFO.SPAWNING_ITEM = true

								local inst_copy = SpawnPrefab(self.item.prefab)

								MOD_ITEMINFO.CACHED_ITEMS[self.item.prefab] = CacheItem(inst_copy)
			--print("Added " .. MOD_ITEMINFO.CACHED_ITEMS[self.item.prefab].prefab .. " to cache")
			
			inst_copy:Remove()
			MOD_ITEMINFO.SPAWNING_ITEM = false
		end
		
		local base_inst = MOD_ITEMINFO.CACHED_ITEMS[self.item.prefab]
		if base_inst.components.equippable and base_inst.components.equippable.dapperness and base_inst.components.equippable.dapperness ~= 0 then
			self:AddDappernessRow(base_inst, self.item)
		end
		
		if base_inst.components.perishable then
			self:AddPerishableRow(base_inst, self.item)
		end
		
		if base_inst.components.insulator then
			self:AddInsulatorRow(base_inst, self.item)
		end
		
		if base_inst.components.waterproofer then
			self:AddWaterproofRow(base_inst, self.item)
		end

		
		if base_inst.components.finiteuses then
			self:AddFiniteUsesRow(base_inst, self.item)
		end
		
		if base_inst.components.weapon then
			self:AddWeaponRow(base_inst, self.item)
		end
		
		if base_inst.components.fueled then
			self:AddFueledRow(base_inst, self.item)
		end
		
		if base_inst.components.armor then
			self:AddArmorRow(base_inst, self.item)
		end
		
		if base_inst.components.healer then
			self:AddHealerRow(base_inst, self.item)
		end
		
		if base_inst.components.edible then
			if 	self:CanEat(base_inst) then
				self:AddEdibleRow(base_inst, self.item)
			end
		end
		
		
	end
	
	if MOD_ITEMINFO.SHOW_PREFABNAME then
		self:AddPrefabNameRow(self.item)
	end
end

function ItemInfoDesc:GetSize()
	return self.width, self.height
end

function ItemInfoDesc:GetWorldSize()
	--local scale = self:GetLooseScale() -- We could use this, but when the widget plays the scale up anim the position is gonna get messed up.
	-- Instead, let's use the scale it's eventually going to have
	return self.width * MOD_ITEMINFO.INFO_SCALE, self.height * MOD_ITEMINFO.INFO_SCALE
end

function ItemInfoDesc:RefreshInfo()
	-- Killing all children and rebuilding the widget is cheaper than indexing the controls, checking if they're up to date and then updating them
	-- ... it's also a lot simpler
	-- Maybe try adding EventListeners for % change? Still rebuild, but only when there's actual change.
	-- And still update. Check if the item has changed/there's no item in the slot.
	-- We'd have to index all values though. And the item.
	-- Need to think about it...
	self:KillAllChildren()
	self:UpdateInfo()
	
	self:Show()
end

function ItemInfoDesc:DoShowAnim()
	if self.anim_done then return end
	
	self:SetScale(self.relative_scale/1.3)
	self:ScaleTo(self.relative_scale/1.3, self.relative_scale, .125)
	
	self.anim_done = true
end

function ItemInfoDesc:ShowInfo()
	if self.shown then return end

	self:KillAllChildren()
	self:UpdateInfo()
	self:DoShowAnim()
	
	self:Show()
end

function ItemInfoDesc:HideInfo()
	self:Hide()
	
	self:KillAllChildren()
end

function ItemInfoDesc:UpdatePos()
	local pos = self.slot:GetWorldPosition()
	local width, height = self:GetWorldSize()
	
	local offsetx, offsety
	offsetx = 0
	offsety = 0

	if not TheInput:ControllerAttached() then
		local tips = GetPlayer().HUD.controls:GetTooltip()
		if tips then
			local scale = GetPlayer().HUD.controls.hover:GetScale()
			local txt_pos = GetPlayer().HUD.controls.hover.text:GetWorldPosition()
			local txt_w,txt_h = GetPlayer().HUD.controls.hover.text:GetRegionSize()
			offsetx = txt_pos.x
			offsety = txt_pos.y + (txt_h+16)*scale.y/2
			self:UpdatePosition(offsetx, offsety)
		end
	else
		local scale = GetPlayer().HUD.controls:GetScale()
		local txt_pos = GetPlayer().HUD.controls.inv.actionstringtitle:GetWorldPosition()
		local txt_w,txt_h = GetPlayer().HUD.controls.inv.actionstringtitle:GetRegionSize()
		offsetx = txt_pos.x
		offsety = txt_pos.y + (txt_h+16)*scale.y/2
		self:UpdatePosition(offsetx, offsety)
	end
end

ItemInfoDesc._StartUpdating = ItemInfoDesc.StartUpdating
function ItemInfoDesc:StartUpdating()
	self:_StartUpdating()
	self.active = true
end

ItemInfoDesc._StopUpdating = ItemInfoDesc.StopUpdating
function ItemInfoDesc:StopUpdating()
	self:_StopUpdating()
	self.active = false
end

-- Kills all its children, hides itself and stops updating next time it updates
function ItemInfoDesc:SetInactive()
	self.active = false
end

function ItemInfoDesc:OnUpdate(dt)
	self.time_since_last_update = self.time_since_last_update + dt
	self.relative_scale = MOD_ITEMINFO.INFO_SCALE * GetPlayer().HUD.controls.bottom_root:GetScale().x
	if not self.active then
		self.time_since_last_update = 0
		self:HideInfo()
		self.anim_done = false
		self:StopUpdating()
		return
	end
	if self.skip_first_fps ~= nil then
		self.skip_first_fps = nil
	else
		self:UpdatePos()
	end
	if self.time_since_last_update > MOD_ITEMINFO.SLOT_UPDATE_TIME then
		self.time_since_last_update = 0
		if self.slot and self.slot.tile and self.slot.tile.item then
			self.item = self.slot.tile.item
			self:KillAllChildren()
			self:UpdateInfo()
			if TheInput:ControllerAttached() and not GetPlayer().HUD.controls.inv.actionstring.shown then
				self:Hide()
			else
				self:Show()
			end
			if not self.anim_done then
				self:DoShowAnim()
			end
		else
			self.anim_done = false
			self:KillAllChildren()
		end
	end
end

return ItemInfoDesc
